searchData={"content_type":"text/markdown","items":[{"doc":"Generates boilerplate for a new Rustler project.\n\nUsage:\n\n```\nmix rustler.new [--module  ] [--name  ] [--otp-app  ]\n```","ref":"Mix.Tasks.Rustler.New.html","title":"mix rustler.new","type":"task"},{"doc":"","ref":"Mix.Tasks.Rustler.New.html#run/1","title":"Mix.Tasks.Rustler.New.run/1","type":"function"},{"doc":"Provides compile-time configuration for a NIF module.\n\nWhen used, Rustler expects the `:otp_app` as option.\nThe `:otp_app` should point to the OTP application that\nthe dynamic library can be loaded from.\n\nFor example:\n\n    defmodule MyNIF do\n      use Rustler, otp_app: :my_nif\n    end\n\nThis allows the module to be configured like so:\n\n    config :my_nif, MyNIF,\n      crate: :my_nif,\n      load_data: [1, 2, 3]","ref":"Rustler.html","title":"Rustler","type":"module"},{"doc":"* `:cargo` - Specify how to envoke the rust compiler. Options are:\n      - `:system` (default) - use `cargo` from the system (must be in `$PATH`)\n      - `{:system,  }` - use `cargo` from the system with the given channel.\n        Specified as a string, passed directly to `cargo` (e.g. \"+nightly\").\n      - `{:rustup,  }` - use `rustup` to specify which channel to use.\n        Available options include: `:stable`, `:beta`, `:nightly`, or a string\n        which specifies a specific version (e.g. `\"1.39.0\"`).\n      - `{:bin, \"/path/to/binary\"}` - provide a specific path to `cargo`.\n\n  * `:crate` - the name of the Rust crate, if different from your `otp_app`\n    value. If you have more than one crate in your project, you will need to\n    be explicit about which crate you intend to use.\n\n  * `:default_features` - a boolean to specify whether the crate's default features\n    should be used.\n\n  * `:env` - Specify a list of environment variables when envoking the compiler.\n\n  * `:features` - a list of features to enable when compiling the crate.\n\n  * `:load_data` - Any valid term. This value is passed into the NIF when it is\n    loaded (default: `0`)\n\n  * `:load_data_fun` - `{Module, :function}` to dynamically generate `load_data`.\n    Default value: `nil`.\n\n    This parameter is mutually exclusive with `load_data`\n    which means that `load_data` has to be set to it's default value.\n\n    Example\n\n        defmodule NIF do\n          use Rustler, load_data_fun: {Deployment, :nif_data}\n        end\n\n        defmodule Deployment do\n          def nif_data do\n            :code.priv_dir(:otp_app) |> IO.iodata_to_binary()\n          end\n        end\n\n  * `:load_from` - This option allows control over where the final artifact should be\n    loaded from at runtime. By default the compiled artifact is loaded from the\n    owning `:otp_app`'s `priv/native` directory. This option comes in handy in\n    combination with the `:skip_compilation?` option in order to load pre-compiled\n    artifacts. To override the default behaviour specify a tuple:\n    `{:my_app, \"priv/native/ \"}`. Due to the way `:erlang.load_nif/2`\n    works, the artifact should not include the file extension (i.e. `.so`, `.dll`).\n\n  * `:mode` - Specify which mode to compile the crate with (default: `:release`)\n\n  * `:path` - By default, rustler expects the crate to be found in `native/ ` in the\n    root of the project. Use this option to override this.\n\n  * `:skip_compilation?` - This option skips envoking the rust compiler. Specify this option\n    in combination with `:load_from` to load a pre-compiled artifact.\n\n  * `:target` - Specify a compile [target] triple.\n\n  * `:target_dir` - Override the compiler output directory.\n\nAny of the above options can be passed directly into the `use` macro like so:\n\n    defmodule MyNIF do\n      use Rustler,\n        otp_app: :my_nif,\n        crate: :some_other_crate,\n        load_data: :something\n    end\n\n[target]: https://doc.rust-lang.org/stable/rustc/platform-support.html","ref":"Rustler.html#module-configuration-options","title":"Configuration options - Rustler","type":"module"},{"doc":"Supported NIF API versions.","ref":"Rustler.html#nif_versions/0","title":"Rustler.nif_versions/0","type":"function"},{"doc":"# Changelog\n\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\nSee [`UPGRADE.md`](./UPGRADE.md) for additional help when upgrading to newer versions.\n\n## [0.30.0] - 2023-10-11\n\n#","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"* Return `Result ` from all `send` functions (#239, #563)\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"* Drop support for `RUSTLER_NIF_VERSION`\n* Deprecate `:rustler_crates` project configuration\n* Mark `use Rustler` module configuration as compile-time\n* Bump Rust edition to 2021\n* Make `:rustler` a compile-time-only dependency (#516, #559)\n* Use `enif_term_type` to implement `Term::get_type` (#538). Please check the\n  [Upgrade Documentation](./UPGRADE.md) for necessary code changes.\n* Raise default NIF version to 2.15\n\n## [0.29.1] - 2023-06-30\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"* Exclude directories from external resources for compatibility with Elixir 1.15\n  (#548, thanks @adrienmo)\n* Fix `NifTaggedEnum` derived `Encoder` impl for named-field variants (#547,\n  thanks @dylanburati)\n* Remove `cfg!` directives in build.rs causing cross-compilation to fail (#555,\n  thanks @fabriziosestito)\n\n## [0.29.0] - 2023-06-22\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"* `ErlOption ` to provide an ergonomic option type for Erlang (#507, thanks @tatsuya6502)\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"* Use Cargo features to define the NIF version level (#537), deprecating\n  `RUSTLER_NIF_VERSION`\n\n## [0.28.0] - 2023-04-24\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"* Support OTP 26 (#526, thanks @philss)\n* Support tuples in NIF macro (#520, #527, thanks @denumerate and @philss)\n* Support for `load_data_fun` to compute `load_data` at runtime (#413, thanks @kaaboaye)\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"* Enhanced NIF macro error messages for invalid attributes (#525, thanks @philss)\n\n## [0.27.0] - 2023-01-17\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"* `MIX_ENV` is no longer considered for determining the build profile. Now, the\n  profile defaults to `:release`. Use the `:mode` option to pick another\n  profile explicitly. (#496)\n\n#","ref":"changelog.html#breaking","title":"BREAKING - Changelog","type":"extras"},{"doc":"* `ResourceArc::make_binary` for safe use of `enif_make_resource_binary` (#487)\n* `OwnedBinary` is now `Sync` (#493)\n* Specified MSRV to be 1.56.1.\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"* Documentation for `load` (#501, thanks @ishitatsuyuki)\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"* Edition 2021 for the rustler mix template (#512, thanks @ayrat555)\n\n## [0.26.0] - 2022-09-02\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"##","ref":"changelog.html#highlight","title":"Highlight - Changelog","type":"extras"},{"doc":"We added `TaggedEnum`, which is a generalized enum type (#440, thanks to @SeokminHong!). Example:\n\n```rust\n#[derive(NifTaggedEnum)]\npub enum TaggedEnum1 {\n    Named { x: i32, y: i32 },\n    String1(String),\n    String2(String),\n    Untagged,\n}\n```\n\nOn the Elixir side, the variants are represented as two-tuples `{tag::atom(),\ninner::term()} | atom()`, where the `inner` term is\n\n* a map for the variant `Named` in the example above\n* a binary for the `String1` and `String2` variants\n\nThe `Untagged` variant is represented as the atom `:untagged` in Elixir.\n\n#","ref":"changelog.html#taggedenum","title":"TaggedEnum - Changelog","type":"extras"},{"doc":"* Added `Clone` and `Copy` for `TermType` (#476, thanks @dvic)\n* Added `Env.whereis_pid()` (#456, thanks @Qqwy)\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"* Use `&[impl Encoder]` for keys and values in `map_from_arrays()` to improve ergonomics (#453, thanks @SeokminHong)\n* Improved encode/decode performance of TaggedEnum considerably (#482, thanks @cleaton)\n* Test on OTP 25 (#455)\n\n#","ref":"changelog.html#improved","title":"Improved - Changelog","type":"extras"},{"doc":"* Lifetime handling in `rustler_codegen` (#483, thanks @turion @SeokminHong and @neosimsim)\n* Support multiple variants with same field names in TaggedEnum (#482, thanks @cleaton)\n* Support .toml file extension for cargo config (#468, thanks @joshuataylor for the report in #467)\n* Disambiguate `encode`/`decode` in generated code (#466, thanks @SeokminHong)\n* Migrate CI to `erlef/setup-beam` (#457, thanks @SeokminHong)\n* Documentation of the `schedule` flag for `nif` macro (#444)\n* Improve documentation (#429, thanks @turion)\n\n## [0.25.0] - 2022-04-11\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"* `NewBinary` now also available as `rustler::NewBinary` (thanks @ayrat555)\n* `Term::map_from_pairs()` to conveniently build a map from a list -of pairs (thanks @philss)\n* CI now also tests against macos\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"* Snake-case warening for auto-generated `RUSTLER_{}_field_{}` variables (renamed to `rustler_{}_field_{}`)\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"* Abort compilation on macos if macos target configuration is missing\n\n## [0.24.0] - 2022-02-24\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"* A `NewBinary` type to create binaries in Rust without going through\n  `OwnedBinary`. This can improve performance. Thanks @dlesl!\n* `TermType` derives `Eq` and `PartialEq`.\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"* `rustler_mix`: Bumped required toml dependency to 0.6\n* Bumped `rustler_sys` dependency to `~2.2`.\n\n#","ref":"changelog.html#updated","title":"Updated - Changelog","type":"extras"},{"doc":"* Rustler supports the latest 3 versions of Elixir and OTP. Currently, those\n  are Elixir => 1.11 and OTP >= 22.\n\n#","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"* Set library file extension based on the compile target, thanks @cocoa-xu!\n* Relaxed Jason version requirement to ~> 1.0\n* Various typos in the documentation, thanks @kianmeng!\n\n## [0.23.0] - 2021-12-22\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- `NifException` for using Elixir exception structs\n- Hashing for term\n- Hash and Equality for `Binary` and `OwnedBinary`\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- Rustler changed its supported range of OTP and Elixir versions. We aim to support the three newest versions of OTP and Elixir.\n- The decoder for `Range` requires that `:step` equals `1`. The `:step` field was introduced with\n  Elixir v1.12 and cannot be represented with Rust's `RangeInclusive`.\n- NIF API bindings are generated using Rust","ref":"changelog.html#changed","title":"Changed - Changelog","type":"extras"},{"doc":"- `mix rustler.new` with Elixir v1.13\n- Template config for `macos`\n- Crash if metadata cannot be retrieved while compiling (#398)\n\n## [0.22.2] - 2021-10-07\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- Fixed a regression introduced with #386: `Rustler.Compiler.Config` called into `cargo` when `skip_compilation?` was set, breaking setups where cargo is not installed. Fixed with #389, thanks @karolsluszniak\n\n## [0.22.1] - 2021-10-05\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- [Breaking change] codegen-generated decoders always raise an error instead of\n  causing the calling NIF to return an atom in some cases\n- Fix codegen problem for untagged enums (#370)\n- Fix handling local dependencies with `@external_resources` (#381)\n\n## [0.22.0] - 2021-06-22\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- Simple `Debug` impl for `rustler::Error`\n- Support newtype and tuple structs for `NifTuple` and `NifRecord`\n- `rustler::Error::Term` encoding an arbitrary boxed encoder, returning `{:error, term}`\n- Generic encoder/decoder for `HashMap `, where `T: Decoder` and `U: Decoder`\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- Compilation time of generated decoders has been reduced significantly.\n- Fixed a segfault caused by `OwnedEnv::send_and_clear`\n\n#","ref":"changelog.html#fixed","title":"Fixed - Changelog","type":"extras"},{"doc":"- Renamed `Pid` to `LocalPid` to clarify that it can't point to a remote process\n- Dependencies have been updated.\n- Derive macros have been refactored.\n- Macros have been renamed and old ones have been deprecated:\n  - `rustler_export_nifs!` is now `rustler::init!`\n  - `rustler_atoms!` is now `rustler::atoms!`\n  - `resource_struct_init!` is now `rustler::resource!`\n- New `rustler::atoms!` macro removed the `atom` prefix from the name:\n\n```rust\n//\n// Before\n//\nrustler::rustler_atoms! {\n    atom ok;\n    atom error;\n    atom renamed_atom = \"Renamed\";\n}\n\n//\n// After\n//\nrustler::atoms! {\n    ok,\n    error,\n    renamed_atom = \"Renamed\",\n}\n```\n\n- NIF functions can be initialized with a simplified syntax:\n\n```rust\n//\n// Before\n//\nrustler::rustler_export_nifs! {\n    \"Elixir.Math\",\n    [\n        (\"add\", 2, add)\n    ],\n    None\n}\n\n//\n// After\n//\nrustler::init!(\"Elixir.Math\", [add]);\n```\n\n- NIFs can be derived from regular functions, if the arguments implement `Decoder` and the return type implements `Encoder`:\n\n```rust\n//\n// Before\n//\nfn add (env: Env , args: &[Term ]) -> Result , Error> {\n    let num1: i64 = args[0].decode()?;\n    let num2: i64 = args[1].decode()?;\n\n    Ok((atoms::ok(), num1 + num2).encode(env))\n}\n\n//\n// After\n//\n#[rustler::nif]\nfn add(a: i64, b: i64) -> i64 {\n  a + b\n}\n```\n\n- `rustler::nif` exposes more options to configure a NIF were the NIF is defined:\n\n```rust\n\n#[rustler::nif(schedule = \"DirtyCpu\")]\npub fn dirty_cpu() -> Atom {\n    let duration = Duration::from_millis(100);\n    std::thread::sleep(duration);\n\n    atoms::ok()\n}\n\n#[rustler::nif(name = \"my_add\")]\nfn add(a: i64, b: i64) -> i64 {\n  a + b\n}\n```\n\n#","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"The rustler compiler has been deprecated and will be removed with v1.0. NIFs\nare no longer defined in `mix.exs`, but are configured with `use Rustler`.  See\nthe documentation for the `Rustler` module. To migrate to the new\nconfiguration:\n\n* Drop `:rustler` from the `:compilers` key in your `mix.exs` `project/0` function\n* Drop `:rustler_crates` from `project/0` and move the configurations into the `use Rustler`\n  of your NIF module or application config:\n\n  ```elixir\n  # config/dev.exs\n  config :my_app, MyApp.Native,\n    mode: :debug\n  ```\n\nFor more information, see [the documentation](https://hexdocs.pm/rustler/0.22.0-rc.1/Rustler.html#module-configuration-options).\n\n## [0.21.0] - 2019-09-07\n\n#","ref":"changelog.html#deprecations","title":"Deprecations - Changelog","type":"extras"},{"doc":"- Support for OTP22.\n- Rust linting with [clippy](https://github.com/rust-lang/rust-clippy).\n- Support for decoding IOLists as binaries, `Term::decode_as_binary`.\n\n#","ref":"changelog.html#added","title":"Added - Changelog","type":"extras"},{"doc":"- `rustler_codegen` is now reexported by the `rustler` crate. Depending on the `rustler_codegen` crate is deprecated.\n- `erlang_nif-sys` has been renamed to `rustler_sys` and vendored into the rustler repo.\n- Replaced the hand-rolled TOML parser in `rustler_mix` with the `toml-elixir` package.\n- Improve error messages for derived encoders/decoders.\n- Rust `bool` now corresponds only to booleans (`false`, `true`) in Elixir. Previously, `nil` and `false` were both decodable to\n  `bool`. To use the previous behaviour, a `Truthy` newtype was introduced.","ref":"changelog.html#changes","title":"Changes - Changelog","type":"extras"},{"doc":"# Upgrade\n\nThis document is intended to simplify upgrading to newer versions by extending the changelog.","ref":"upgrade.html","title":"Upgrade","type":"extras"},{"doc":"1. `rustler_crates` configuration is deprecated in favor of explicitly passing\n   options on `use Rustler` or configuring the module in your `config/*.exs`\n   files.\n\n2. `Env::send` and `OwnedEnv::send_and_clear` will now return a `Result`.\n   Updating will thus introduce warnings about unused `Result`s. To remove the\n   warnings without changing behaviour, the `Result`s can be \"used\" as\n   ```rust\n   let _ = env.send(...)\n   ```\n   Neither the `Ok` nor the `Err` case carry additional information so far. An\n   error is returned if either the receiving or the sending process is dead.\n   See also\n   [enif\\_send](https://www.erlang.org/doc/man/erl_nif.html#enif_send).\n\n3. As `Term::get_type` is now implemented using `enif_get_type`, some cases of\n   the `TermType` `enum` are changed, removed, or added:\n   1. `EmptyList` is dropped, `List` is returned for both empty and non-empty\n      lists\n   2. `Exception` is dropped\n   3. `Number` is split into `Integer` and `Float` (if NIF 2.14 support is\n      explicitly enforced, only `Float` is returned)\n\n4. The default NIF version is raised to 2.15 to make use of `enif_get_type`. To\n   use a compiled NIF with an older version than OTP22, disable the default\n   features and expliictly use the `nif_version_2_14` feature in the library's\n   `Cargo.toml`:\n   ```toml\n   rustler = { version = \"0.30\", default-features = false, features = [\"derive\", \"nif_version_2_14\"] }\n   ```\n\n5. As noted for the `0.28 -> 0.29` transition below, the environment variable\n   `RUSTLER_NIF_VERSION` will not be considered anymore from 0.30 onwards.","ref":"upgrade.html#0-29-0-30","title":"0.29 -> 0.30 - Upgrade","type":"extras"},{"doc":"`RUSTLER_NIF_VERSION` is deprecated and will not be considered anymore for 0.30.\nThe NIF version will also not be guessed anymore from a potentially available\ninstalled Erlang version. By default, NIF libraries will now be compiled against\nNIF version 2.14 which is compatible down to OTP21. The default will be adjusted\nalong with the supported OTP versions.\n\nIf additional features are required that use newer NIF versions, these can be\nincluded explicitly in the project's `Cargo.toml` as, e.g.\n\n```toml\n[dependencies]\nrustler = { version = \"0.30\", features = [\"nif_version_2_17\"] }\n```\n\nWith this configuration, the resulting NIF library will only work from OTP26\nonwards, but will also have access to the largest feature set.","ref":"upgrade.html#0-28-0-29","title":"0.28 -> 0.29 - Upgrade","type":"extras"},{"doc":"`MIX_ENV` is no longer considered for determining the build profile. Now, the\nprofile defaults to `:release`. Use the `:mode` option to pick another profile\nexplicitly. See #496.","ref":"upgrade.html#0-26-0-27","title":"0.26 -> 0.27 - Upgrade","type":"extras"},{"doc":"0.22 changes how to define NIFs. Users upgrading to 0.22 should to do these things:\n\n1. Replace `rustler_atoms!` with `rustler::atoms!`\n2. Replace `resource_struct_init!` with `rustler::resource!`\n3. Replace `rustler::rustler_export_nifs!` with `rustler::init!`\n4. Use the new `rustler::nif` proc_macro to declare NIFs\n\nReplacing `rustler_atoms!` with `rustler::atoms!` is fairly simple and already\nsufficiently described in [CHANGELOG.md](./CHANGELOG.md). Similarly, replacing\n`resource_struct_init!` with `rustler::resource!` is a simple rename, so this does\nnot need additional examples here.\n\n#","ref":"upgrade.html#0-21-0-22","title":"0.21 -> 0.22 - Upgrade","type":"extras"},{"doc":"`rustler::init!` in combination with the new `rustler::nif` proc_macro\nsimplifies exporting NIFs. Before, the NIFs and their arity needed to be specified\nusing tuple syntax:\n\n```rust\nrustler::rustler_export_nifs! {\n    \"Elixir.Math\",\n    [\n        (\"add\", 2, add),\n\t(\"long_running_operation\", 0, long_running_operation, SchedulerFlags::DirtyCpu)\n    ],\n    None\n}\n```\n\nNow, listing the NIFs directly is sufficient:\n\n```rust\nrustler::init!(\"Elixir.Math\", [add, long_running_operation]);\n```\n\nWith this new macro, defining an `on_load` function (e.g. to set up a resource with\n`rustler::resource!`), is done like this:\n\n```rust\nrustler::init!(\"Elixir.Math\", [add, long_running_operation], load = a_function);\n```\n\nNote that NIF flags such as `SchedulerFlags::DirtyCpu` are not declared in `rustler::init!`, but\nusing the proc_macro `rustler::nif`. See further below for information on migration NIF flags.\n\n#","ref":"upgrade.html#replace-rustler-rustler_export_nifs-with-rustler-init","title":"Replace `rustler::rustler_export_nifs!` with `rustler::init!` - Upgrade","type":"extras"},{"doc":"0.22 introduces a new `proc_macro` allowing to spell out the parameter of a NIF\ndirectly instead of using an `args: &[Term ]`. Lets consider an example `add()`,\nwhere the Elixir function looks like this:\n\n```elixir\ndef add(left, right), do: :erlang.nif_error(:not_loaded)\n```\n\nPreviously, the signature of the corresponding NIF might have looked like this:\n\n```rust\nfn add (env: Env , args: &[Term ]) -> Result , Error>\n```\n\nWhen calling the NIF from Elixir as `add(1, 2)`, `args` would then contain two\n`Term`, one for 1, and one for 2. With 0.22, this becomes more obvious, as the\nNIFs signature resembles the Elixir function's signature:\n\n```rust\n#[rustler::nif]\nfn add(a: i64, b: i64) -> i64\n```\n\nUnder the hood, this is implemented by the `rustler::nif` proc_macro. For the\nnew form to work, the parameters' types need to implement `Decoder`, and the\nreturn type needs to implement `Encoder`.\n\n##","ref":"upgrade.html#use-the-new-rustler-nif-proc_macro-to-declare-nifs","title":"Use the new `rustler::nif` proc_macro to declare NIFs - Upgrade","type":"extras"},{"doc":"Sometimes, we still need the environment `Env` for the NIF. For example, if\nwork with `Binary` and `OwnedBinary`, the environment would be needed to create a `Binary`\nfrom an `OwnedBinary`. To allow this, `env: Env ` can be added explicitly as well:\n\n```rust\n#[rustler::nif]\npub fn map_entries_sorted (env: Env , iter: MapIterator ) -> NifResult >>\n```\n\n`env` can then be used the same way as before.\n\n##","ref":"upgrade.html#what-if-env-is-required-in-the-function","title":"What if `Env` is required in the function? - Upgrade","type":"extras"},{"doc":"The `rustler::nif` proc_macro allows setting options directly on a NIF. Assume that we have a\nNIF called `_long_running_operation`, which used to be declared prior to Rustler v0.22 like this:\n\n```rust\n// Before\n\nrustler::rustler_export_nifs! {\n    \"Elixir.SomeNif\",\n    [\n\t// Note that the function in Rust is long_running_operation, but the NIF is exported as\n\t// _long_running_operation!\n\t(\"_long_running_operation\", 0, long_running_operation, SchedulerFlags::DirtyCpu)\n    ],\n    None\n}\n\nfn long_running_operation (env: Env , _args: &[Term ]) -> Result , Error> {\n  // ...\n}\n```\n\nThis definition declares that a function `_long_running_operation` with arity\nzero is to be exported, and that this function should be schedules on the\n`DirtyCpu` scheduler. With the changes in Rustler v0.22, the function would be declared like\nthis:\n\n```rust\n// Now\n\nrustler::init!(\"Elixir.SomeNif\", [long_running_operation]);\n\n#[rustler::nif(\n    rename = \"_long_running_operation\",\n    schedule = \"DirtyCpu\"\n  )]\nfn long_running_operation() -> TheProperReturnType {\n // ..\n}\n```","ref":"upgrade.html#migrating-flags-and-rename","title":"Migrating Flags and Rename - Upgrade","type":"extras"},{"doc":"Apache License\n                        Version 2.0, January 2004\n                     http://www.apache.org/licenses/\n\nTERMS AND CONDITIONS FOR USE, REPRODUCTION, AND DISTRIBUTION\n\n1. Definitions.\n\n   \"License\" shall mean the terms and conditions for use, reproduction,\n   and distribution as defined by Sections 1 through 9 of this document.\n\n   \"Licensor\" shall mean the copyright owner or entity authorized by\n   the copyright owner that is granting the License.\n\n   \"Legal Entity\" shall mean the union of the acting entity and all\n   other entities that control, are controlled by, or are under common\n   control with that entity. For the purposes of this definition,\n   \"control\" means (i) the power, direct or indirect, to cause the\n   direction or management of such entity, whether by contract or\n   otherwise, or (ii) ownership of fifty percent (50%) or more of the\n   outstanding shares, or (iii) beneficial ownership of such entity.\n\n   \"You\" (or \"Your\") shall mean an individual or Legal Entity\n   exercising permissions granted by this License.\n\n   \"Source\" form shall mean the preferred form for making modifications,\n   including but not limited to software source code, documentation\n   source, and configuration files.\n\n   \"Object\" form shall mean any form resulting from mechanical\n   transformation or translation of a Source form, including but\n   not limited to compiled object code, generated documentation,\n   and conversions to other media types.\n\n   \"Work\" shall mean the work of authorship, whether in Source or\n   Object form, made available under the License, as indicated by a\n   copyright notice that is included in or attached to the work\n   (an example is provided in the Appendix below).\n\n   \"Derivative Works\" shall mean any work, whether in Source or Object\n   form, that is based on (or derived from) the Work and for which the\n   editorial revisions, annotations, elaborations, or other modifications\n   represent, as a whole, an original work of authorship. For the purposes\n   of this License, Derivative Works shall not include works that remain\n   separable from, or merely link (or bind by name) to the interfaces of,\n   the Work and Derivative Works thereof.\n\n   \"Contribution\" shall mean any work of authorship, including\n   the original version of the Work and any modifications or additions\n   to that Work or Derivative Works thereof, that is intentionally\n   submitted to Licensor for inclusion in the Work by the copyright owner\n   or by an individual or Legal Entity authorized to submit on behalf of\n   the copyright owner. For the purposes of this definition, \"submitted\"\n   means any form of electronic, verbal, or written communication sent\n   to the Licensor or its representatives, including but not limited to\n   communication on electronic mailing lists, source code control systems,\n   and issue tracking systems that are managed by, or on behalf of, the\n   Licensor for the purpose of discussing and improving the Work, but\n   excluding communication that is conspicuously marked or otherwise\n   designated in writing by the copyright owner as \"Not a Contribution.\"\n\n   \"Contributor\" shall mean Licensor and any individual or Legal Entity\n   on behalf of whom a Contribution has been received by Licensor and\n   subsequently incorporated within the Work.\n\n2. Grant of Copyright License. Subject to the terms and conditions of\n   this License, each Contributor hereby grants to You a perpetual,\n   worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n   copyright license to reproduce, prepare Derivative Works of,\n   publicly display, publicly perform, sublicense, and distribute the\n   Work and such Derivative Works in Source or Object form.\n\n3. Grant of Patent License. Subject to the terms and conditions of\n   this License, each Contributor hereby grants to You a perpetual,\n   worldwide, non-exclusive, no-charge, royalty-free, irrevocable\n   (except as stated in this section) patent license to make, have made,\n   use, offer to sell, sell, import, and otherwise transfer the Work,\n   where such license applies only to those patent claims licensable\n   by such Contributor that are necessarily infringed by their\n   Contribution(s) alone or by combination of their Contribution(s)\n   with the Work to which such Contribution(s) was submitted. If You\n   institute patent litigation against any entity (including a\n   cross-claim or counterclaim in a lawsuit) alleging that the Work\n   or a Contribution incorporated within the Work constitutes direct\n   or contributory patent infringement, then any patent licenses\n   granted to You under this License for that Work shall terminate\n   as of the date such litigation is filed.\n\n4. Redistribution. You may reproduce and distribute copies of the\n   Work or Derivative Works thereof in any medium, with or without\n   modifications, and in Source or Object form, provided that You\n   meet the following conditions:\n\n   (a) You must give any other recipients of the Work or\n       Derivative Works a copy of this License; and\n\n   (b) You must cause any modified files to carry prominent notices\n       stating that You changed the files; and\n\n   (c) You must retain, in the Source form of any Derivative Works\n       that You distribute, all copyright, patent, trademark, and\n       attribution notices from the Source form of the Work,\n       excluding those notices that do not pertain to any part of\n       the Derivative Works; and\n\n   (d) If the Work includes a \"NOTICE\" text file as part of its\n       distribution, then any Derivative Works that You distribute must\n       include a readable copy of the attribution notices contained\n       within such NOTICE file, excluding those notices that do not\n       pertain to any part of the Derivative Works, in at least one\n       of the following places: within a NOTICE text file distributed\n       as part of the Derivative Works; within the Source form or\n       documentation, if provided along with the Derivative Works; or,\n       within a display generated by the Derivative Works, if and\n       wherever such third-party notices normally appear. The contents\n       of the NOTICE file are for informational purposes only and\n       do not modify the License. You may add Your own attribution\n       notices within Derivative Works that You distribute, alongside\n       or as an addendum to the NOTICE text from the Work, provided\n       that such additional attribution notices cannot be construed\n       as modifying the License.\n\n   You may add Your own copyright statement to Your modifications and\n   may provide additional or different license terms and conditions\n   for use, reproduction, or distribution of Your modifications, or\n   for any such Derivative Works as a whole, provided Your use,\n   reproduction, and distribution of the Work otherwise complies with\n   the conditions stated in this License.\n\n5. Submission of Contributions. Unless You explicitly state otherwise,\n   any Contribution intentionally submitted for inclusion in the Work\n   by You to the Licensor shall be under the terms and conditions of\n   this License, without any additional terms or conditions.\n   Notwithstanding the above, nothing herein shall supersede or modify\n   the terms of any separate license agreement you may have executed\n   with Licensor regarding such Contributions.\n\n6. Trademarks. This License does not grant permission to use the trade\n   names, trademarks, service marks, or product names of the Licensor,\n   except as required for reasonable and customary use in describing the\n   origin of the Work and reproducing the content of the NOTICE file.\n\n7. Disclaimer of Warranty. Unless required by applicable law or\n   agreed to in writing, Licensor provides the Work (and each\n   Contributor provides its Contributions) on an \"AS IS\" BASIS,\n   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or\n   implied, including, without limitation, any warranties or conditions\n   of TITLE, NON-INFRINGEMENT, MERCHANTABILITY, or FITNESS FOR A\n   PARTICULAR PURPOSE. You are solely responsible for determining the\n   appropriateness of using or redistributing the Work and assume any\n   risks associated with Your exercise of permissions under this License.\n\n8. Limitation of Liability. In no event and under no legal theory,\n   whether in tort (including negligence), contract, or otherwise,\n   unless required by applicable law (such as deliberate and grossly\n   negligent acts) or agreed to in writing, shall any Contributor be\n   liable to You for damages, including any direct, indirect, special,\n   incidental, or consequential damages of any character arising as a\n   result of this License or out of the use or inability to use the\n   Work (including but not limited to damages for loss of goodwill,\n   work stoppage, computer failure or malfunction, or any and all\n   other commercial damages or losses), even if such Contributor\n   has been advised of the possibility of such damages.\n\n9. Accepting Warranty or Additional Liability. While redistributing\n   the Work or Derivative Works thereof, You may choose to offer,\n   and charge a fee for, acceptance of support, warranty, indemnity,\n   or other liability obligations and/or rights consistent with this\n   License. However, in accepting such obligations, You may act only\n   on Your own behalf and on Your sole responsibility, not on behalf\n   of any other Contributor, and only if You agree to indemnify,\n   defend, and hold each Contributor harmless for any liability\n   incurred by, or claims asserted against, such Contributor by reason\n   of your accepting any such warranty or additional liability.\n\nEND OF TERMS AND CONDITIONS\n\nAPPENDIX: How to apply the Apache License to your work.\n\n   To apply the Apache License to your work, attach the following\n   boilerplate notice, with the fields enclosed by brackets \"[]\"\n   replaced with your own identifying information. (Don't include\n   the brackets!)  The text should be enclosed in the appropriate\n   comment syntax for the file format. We also recommend that a\n   file or class name and description of purpose be included on the\n   same \"printed page\" as the copyright notice for easier\n   identification within third-party archives.\n\nCopyright [yyyy] [name of copyright owner]\n\nLicensed under the Apache License, Version 2.0 (the \"License\");\nyou may not use this file except in compliance with the License.\nYou may obtain a copy of the License at\n\n\thttp://www.apache.org/licenses/LICENSE-2.0\n\nUnless required by applicable law or agreed to in writing, software\ndistributed under the License is distributed on an \"AS IS\" BASIS,\nWITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\nSee the License for the specific language governing permissions and\nlimitations under the License.","ref":"license-apache.html","title":"License (Apache-2.0)","type":"extras"},{"doc":"Copyright (c) 2016 hansihe\n\nPermission is hereby granted, free of charge, to any\nperson obtaining a copy of this software and associated\ndocumentation files (the \"Software\"), to deal in the\nSoftware without restriction, including without\nlimitation the rights to use, copy, modify, merge,\npublish, distribute, sublicense, and/or sell copies of\nthe Software, and to permit persons to whom the Software\nis furnished to do so, subject to the following\nconditions:\n\nThe above copyright notice and this permission notice\nshall be included in all copies or substantial portions\nof the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF\nANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED\nTO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A\nPARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT\nSHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY\nCLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION\nOF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR\nIN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER\nDEALINGS IN THE SOFTWARE.","ref":"license-mit.html","title":"License (MIT)","type":"extras"},{"doc":"# Rustler\n\n[![Module Version](https://img.shields.io/hexpm/v/rustler.svg)](https://hex.pm/packages/rustler)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/rustler/)\n[![Total Download](https://img.shields.io/hexpm/dt/rustler.svg)](https://hex.pm/packages/rustler)\n[![License](https://img.shields.io/hexpm/l/rustler.svg)](https://github.com/rusterlium/rustler/blob/master/LICENSE)\n[![Last Updated](https://img.shields.io/github/last-commit/rusterlium/rustler.svg)](https://github.com/rusterlium/rustler/commits/master)\n\nThe Mix package for [rustler](https://github.com/rusterlium/rustler), a library to write Erlang Native Implemented Functions (NIFs) in [Rust](https://www.rust-lang.org/) programming language.","ref":"readme.html","title":"Rustler","type":"extras"},{"doc":"This package is available on [Hex.pm](https://hex.pm/packages/rustler). To install it, add `:rustler` to your dependencies:\n\n```elixir\ndef deps do\n  [\n    {:rustler, \"~> 0.30.0\", runtime: false}\n  ]\nend\n```","ref":"readme.html#installation","title":"Installation - Rustler","type":"extras"},{"doc":"1.  Fetch all necessary dependencies:\n\n    ```\n    $ mix deps.get\n    ```\n2.  Check your installation by showing help from the installed Mix task:\n\n    ```\n    $ mix help rustler.new\n    ```\n\n3.  Generate the boilerplate for a new Rustler project. Follow the instructions\n    to configure your project:\n\n    ```\n    $ mix rustler.new\n    ```\n\n4.  [Load the NIF in your program.](#loading-the-nif).","ref":"readme.html#usage","title":"Usage - Rustler","type":"extras"},{"doc":"The Rust crate compilation can be controlled via Mix compile-time configuration in `config/config.exs`.\nSee [configuration options](https://hexdocs.pm/rustler/Rustler.html#module-configuration-options) for more details.","ref":"readme.html#crate-configuration","title":"Crate configuration - Rustler","type":"extras"},{"doc":"Loading a Rustler NIF is done in almost the same way as normal NIFs.\n\nThe actual loading is done by calling `use Rustler, otp_app: :my_app` in the module you want to load the NIF in.\nThis sets up the `@on_load` module hook to load the NIF when the module is first\nloaded.\n\n```elixir\ndefmodule MyProject.MyModule do\n  use Rustler,\n    otp_app: :my_app,\n    crate: :my_crate\n\n  # When loading a NIF module, dummy clauses for all NIF function are required.\n  # NIF dummies usually just error out when called when the NIF is not loaded, as that should never normally happen.\n  def my_native_function(_arg1, _arg2), do: :erlang.nif_error(:nif_not_loaded)\nend\n```\n\nNote that `:crate` is the name in the `[lib]` section of your `Cargo.toml`. The\n`:crate` option is optional if your crate and `otp_app` use the same name.\n\nSee the `Rustler` module for more information.","ref":"readme.html#loading-the-nif","title":"Loading the NIF - Rustler","type":"extras"},{"doc":"Licensed under either of\n\n- Apache License, Version 2.0, ([LICENSE-APACHE](../LICENSE-APACHE) or http://www.apache.org/licenses/LICENSE-2.0)\n- MIT license ([LICENSE-MIT](../LICENSE-MIT) or http://opensource.org/licenses/MIT)\n\nat your option.","ref":"readme.html#copyright-and-license","title":"Copyright and License - Rustler","type":"extras"},{"doc":"Unless you explicitly state otherwise, any contribution intentionally submitted for inclusion in the work by you, as defined in the Apache-2.0 license, shall be dual licensed as above, without any additional terms or conditions.","ref":"readme.html#contribution","title":"Contribution - Rustler","type":"extras"}]}